"spdest" <-
function(price,rfr=0,mat=1,ttime=NULL,fast=FALSE,fast.type="brab",nrec=2,iter=2)
{
############################
## price: (n x 2) matrix;
##   1-st column strike price, 2-nd column option price, 3-rd column put indicator (0-1)
## rfr: risk-free interest rate
## mat: time to maturity in years
## ttime: vector of length n;
##   if given, "time" of the transaction is used to calculate the covariance structure
## fast: fast or exact calculation?
## fast.type: type of the fast algorithm (trace,det,trace2,alg)
##   tr: keep only most recent obs
##   det: the means calculated using optimal weights for determinant
##   crab: most recent obs + close neighbour in ttime
##   nrec: number of the most recent observations kept within each block
##   iter: number of iterations for selecting close observations in adjacent blocks (CRAB)
############################
## check parameter price  
  if(!is.matrix(price)) stop("spdest: price has to be matrix")
  if(dim(price)[2]!=3) stop("spdest: matrix price does not have 3 columns")
  n=nrow(price)
## check parameters rfr and mat
  if(!is.numeric(rfr)) stop("spdest: parameter rfr is not numeric")
  if(!is.numeric(mat)) stop("spdest: parameter mat is not numeric")
  if(length(rfr)!=1) stop("spdest: rfr must be scalar")
  if(length(mat)!=1) stop("spdest: mat must be scalar")
  if(length(nrec)!=1) stop("spdest: nrec must be scalar")
  if(length(iter)!=1) stop("spdest: iter must be scalar")
## check parameter ttime
  TT=TRUE
  if(length(ttime)==0) {
    ttime=rep(1,n)
    TT=FALSE
  }
  if(!is.numeric(ttime)) stop("spdest: parameter ttime is not numeric")
  ttime=as.vector(ttime)
  if(length(ttime)!=n) stop("spdest: parameter ttime has wrong length")
## 
  if(!is.logical(fast)) stop("spdest: parameter fast is not logical")
##i/ correct the observed prices by the risk-free interest rate and time to maturity 
  price[,2]<-price[,2]*exp(rfr*mat)
##ii/ order everything by: call & put & strike & ttime?
  ord01=order(price[,3],price[,1],ttime)
  price=price[ord01,]
  ttime=ttime[ord01]
##1/ find out if we have only "put", only "call", or both option types
##   and create logicals (C and P) for future control flow of the algorithm
  CL=FALSE
  PT=FALSE
  puts=as.logical(price[,3])
  np=sum(puts)
  nc=n-np
  PT=as.logical(np)
  CL=as.logical(nc)
  if((!CL)&&(!PT)) stop("spdest: unexpected problem with put indicator")
##2/ create the dummy variables (separately for C and P)
## now the design matrices...
  tmp=spdest.xd(price)
  L=tmp$L
  K=tmp$K
  xc=tmp$xc
  xp=tmp$xp
  y=tmp$y
  xd=tmp$xd
###########################################################
##3/ calculate the covariance matrix of option prices (depending on C and P)
##   and on the chosen "fast" algorithm
###########################################################  
  if(fast) {
## we have: price, cvr, ttime, y, xd (nc calls followed by np puts)
#    print(fast.type)
    if(!is.element(fast.type,c("tr","det","crab","brab"))) {
      warning("spdest: unrecognized value of parameter fast.type")
    }
    if(fast.type=="crab"|fast.type=="brab"|fast.type=="tr") {
      recent.cl=numeric(0)
      recent.pt=numeric(0)
      if(CL) recent.cl=spdest.recent(ttime[!puts],cumsum(1-duplicated(xd[!puts,])),fast.type,nrec,iter)$select
      if(PT) recent.pt=nc+spdest.recent(ttime[puts],cumsum(1-duplicated(xd[puts,])),fast.type,nrec,iter)$select
      recent=sort(unique(round(c(recent.cl,recent.pt))))
      price=price[recent,]
      xd=xd[recent,]
      ttime=ttime[recent]
      y=y[recent]
    }
    if(fast.type=="det") {
      ## separately for calls and puts: can be improved later :-)
      w=numeric(0)
      if(CL) {
        w=spdest.fastw(price[1:nc,1],ttime[1:nc])$w
      }
      if(PT) {
        wp=spdest.fastw(price[(nc+1):(nc+np),1],ttime[(nc+1):(nc+np)])$w
        w=rbind(cbind(w,matrix(0,NROW(w),np)),cbind(matrix(0,nrow(wp),nc),wp))
      }
      ##3/ calculate the covariance matrix of option prices (depending on C and P) for "det"
      if(!TT) {
        ## assume iid and heteroskedasticity if the transaction times are not given
        cvr=diag(diag(xd%*%t(xd)))
      } else {
        ## spdest.cvr calculates the covariance structure
        cvr=spdest.cvr(ttime,L,xd,nc,np,CL,PT)
      }
      xd=unique(xd)
      y=w%*%as.matrix(y)
      cvr=w%*%cvr%*%t(w)
      ttime=w%*%ttime
    }    
    if(fast.type!="det") {
      if (PT) {
        np=sum(xd[,NCOL(xd)])
      } else {
        np=0
      }
      nc=NROW(xd)-np
      if(!TT) {
       ## assume iid and heteroskedasticity if the transaction times are not given
        cvr=diag(diag(xd%*%t(xd)))
      } else {
        ## spdest.cvr calculates the covariance structure
        cvr=spdest.cvr(ttime,L,xd,nc,np,CL,PT)
      }
    }
  } else {
    if(!TT) {
      ## assume iid and heteroskedasticity if the transaction times are not given
      cvr=diag(diag(xd%*%t(xd)))
    } else {
      ## spdest.cvr calculates the covariance structure
      cvr=spdest.cvr(ttime,L,xd,nc,np,CL,PT)
    }
  }
###########################################################
##4/ numerical minimization of the least squares
## L-1 interesting parameters + 1 or 2 intercepts
## y is the response variable, xd is the design matrix
##
## a/ linear regression (for getting reasonable starting values)
  tmp=eigen(cvr) #???+diag(rep(1,sum(keep))))
#  cvr12=tmp$vectors%*%diag(1/sqrt(tmp$values))%*%t(tmp$vectors)
## get rid of too small eigenvalues :-(
  keep=tmp$values>quantile(tmp$values,0.01)
  cvrinv=tmp$vectors[,keep]%*%diag(1/tmp$values[keep])%*%t(tmp$vectors[,keep])
#  start.lm=lm.gls(y~xd-1,W=cvrinv)
  beta.lm=as.vector(ginv(t(xd)%*%cvrinv%*%xd)%*%(t(xd)%*%cvrinv%*%y))
  resi.lm=as.vector(y-xd%*%as.matrix(beta.lm))
  s2.lm=as.numeric(t(as.matrix(resi.lm))%*%cvrinv%*%as.matrix(resi.lm)/(NROW(xd)-NCOL(xd)))
  var.lm=s2.lm*ginv(t(xd)%*%cvrinv%*%xd)
  var.diag.lm=diag(var.lm)[2:(L-1)] #the spd estimates are parameters 2:(L-1)
## b/ reasonable starting value for the algorithm... (applying the constraints on 1-st derivative)
#  beta.lm=as.vector(coef(start.lm))
  der1.lm=1-cumsum(beta.lm[1:(L-1)])[(L-1):1]
  der1.start=isoreg((2:L),pmin(pmax(0,der1.lm),1))$yf
  beta.start=c(c(1,der1.start[(L-1):2])-der1.start[(L-1):1],pmax(beta.lm[L:length(beta.lm)],0))
## c/ beta --> theta
  theta.start=log(pmax(c(spdest.beta2theta(beta.start[1:(L-1)]),beta.start[L:length(beta.start)]),1e-8))
## d/ minimize the nonlinear least squares
  tmp=try(optim(theta.start,spdest.nnls,method="BFGS",control=list(parscale=theta.start,ndeps=rep(1e-6,length(theta.start))),hessian=TRUE,L=L,y=y,xd=xd,cvrinv=cvrinv),TRUE)
  if(inherits(tmp, "try-error")) {
    tmp=try(optim(theta.start,spdest.nnls,method="Nelder-Mead",control=list(parscale=theta.start),hessian=TRUE,L=L,y=y,xd=xd,cvrinv=cvrinv),TRUE)
    warning("BFGS failed. Trying Nelder-Mead.")
  }
  if(inherits(tmp, "try-error")) {
    tmp=try(list(par=theta.start),TRUE)
    warning("Nelder-Mead failed. No iterations.")
#    print(theta.start)
  }
  if(inherits(tmp, "try-error")) {
    stop("Numerical minimization failed.")
  }
  theta.optim=tmp$par
#  theta.hessian=tmp$hessian
  beta.optim=spdest.theta2beta(theta.optim,L)
#5/ variance matrix!
  sigma=spdest.nnls(theta=theta.optim,L=L,y=y,xd=xd,cvrinv=cvrinv)/(NROW(xd)-NCOL(xd))
## correct scale (since the subvector of theta.optim corresp. to beta.optim does not have to sum to one)
  scaling=sum(exp(theta.optim[1:L]))
  var.logspd=diag(1/exp(theta.optim[2:(L-1)]),nrow=L-2)%*%ginv(scaling*t(xd)%*%cvrinv%*%xd)[1:(L-1),1:(L-1)][2:(L-1),2:(L-1)]%*%diag(1/exp(theta.optim[2:(L-1)]),nrow=L-2)*sigma
#  var.logspd=ginv(scaling*theta.hessian[1:(L-1),1:(L-1)])[2:(L-1),2:(L-1)]*sigma
##??  var.spd=(diag(exp(beta.optim[2:(L-1)]))%*%var.logspd%*%diag(exp(beta.optim[2:(L-1)])))
  var.spd=var.logspd+(diag(beta.optim[2:(L-1)],nrow=L-2)%*%var.logspd%*%diag(beta.optim[2:(L-1)],nrow=L-2))
  var.diag=diag(var.spd)
#6/ nice results...  
## standard errors of parameter estimates
  cil.lm=qt(0.975,NROW(xd)-L)*sqrt(var.diag.lm)
  ci.lm=data.frame(low=beta.lm[2:(L-1)]-cil.lm,up=beta.lm[2:(L-1)]+cil.lm,row.names=(L-2):1)
#  cil=1.96*sqrt(var.diag)
  cil=qt(0.975,NROW(xd)-L)*sqrt(diag(var.logspd))
  ci=data.frame(low=beta.optim[2:(L-1)]*exp(-cil),up=beta.optim[2:(L-1)]*exp(cil),row.names=(L-2):1)
## first derivatives
## we already have der1.lm
  der1.optim=1-cumsum(beta.optim[1:(L-1)])[(L-1):1]
## fit the put and call option prices
## create the "short" design matrices
  xd.short=unique(xd)
  K.short=unique(price[,c(1,3)])[,1]
  fit.short=xd.short%*%as.matrix(beta.optim)
  fit.call=NULL
  fit.put=NULL
  if(CL) {
    cals.short=as.logical(xd.short[,L])
    fit.call=data.frame(strike=K.short[cals.short],fit=fit.short[cals.short])
  }
  if(PT) {
    puts.short=as.logical(xd.short[,NCOL(xd.short)])
    fit.short[puts.short]=fit.short[puts.short]-xd.short[puts.short,1]
    fit.put=data.frame(strike=K.short[puts.short],fit=fit.short[puts.short])
  }
  return(list(spd=beta.optim[(L-1):2],var.spd=var.diag[(L-2):1],fit.call=fit.call,fit.put=fit.put,der=der1.optim,ci=ci[(L-2):1,],ci.lm=ci.lm[(L-2):1,],n=length(y)))
}

