"spdest.recent" <-
function(ttime,blocks,fast.type,nrec=2,iter=NA) {
#################################################################
### for crab and brab
#################################################################
### ttime is the trabsactuib time (most recent obs have largest ttime)
### blocks denotes the blocks
### fast type is crab (closest recent obs. in adjacent blocks)
###           or brab (bracket recent obs. in adjacent blocks)
###           or tr   (same as crab with iter=0)
### nrec is "number of recent"
### iter is "number of iterations"
#################################################################
#################################################################
  if(!is.element(fast.type,c("tr","brab","crab"))) {
    warning("spdest: unrecognized value of parameter fast.type")
  }
  if(fast.type=="tr") {
    fast.type=crab
    iter=0
  }
  n=length(blocks)
  L=length(unique(blocks))
  label=paste(fast.type,"(",nrec,",",iter,")",sep="")
  starts=round(which(as.logical(1-duplicated(blocks)))-1)
  blngth=round(tapply(rep(1,n),blocks,sum))

  ## if iter is not given, we will iterate until it does not change or until maxiter
  auto=is.na(iter)
  if(auto) {iter=n}

  ## indeces with largest ttime (most recent) within each block are always needed
  tmp=tapply(ttime,blocks,which.max)+starts
  if(nrec>=2){
    for (i in 2:nrec) {
      ## second most recent etc.
      recent=tapply(ttime[-tmp],factor(blocks[-tmp],levels=unique(blocks),exclude=NULL),which.max)
      recent=replace(recent,is.na(recent),1)+starts
      tmp=sort(unique(round(c(tmp,recent))))
    }
  }
  recent=tmp
  niter=length(recent)
  i=0
  if(fast.type=="crab"&iter>0) {
    ## we have tmp (most recent indeces)
    close=rep(FALSE,n)
    goon=TRUE
    while((i<iter)&(goon)) {
      i=i+1
      nprev=niter
      for(bl in 1:length(starts)) {
        ## look in the neighbouring blocks for closest indeces
        times=ttime[((blocks==(bl-1))|(blocks==(bl+1)))&is.element(1:n,tmp)]
        ## select obs in block closest to each element of times...
        for(t in times) {
          indx=round(which.min(abs(ttime[blocks==bl]-rep(t,blngth[bl])))+starts[bl])
          close[indx]=TRUE
        }
      }
      tmp=sort(unique(round(c(tmp,(1:n)[close[1:n]]))))
      niter=length(tmp)
      if(niter==nprev) {
        ## stop if we don't add any new observations
        goon=FALSE
      }
    }
    recent=sort(unique(round(c(recent,tmp))))
  }
  if(fast.type=="brab"&iter>0) {
    ## we have tmp (most recent indeces)
    bracket=rep(FALSE,n)
    goon=TRUE
    while((i<iter)&(goon)) {
      i=i+1
      nprev=niter
      for(bl in 1:length(starts)) {
        ## which times are already selected in neighbouring blocks
        times=ttime[((blocks==(bl-1))|(blocks==(bl+1)))&is.element(1:n,tmp)]
        ## select obs in current block bracketing each element of times...
        for(t in times) {
          dif=ttime[blocks==bl]-rep(t,blngth[bl])
          top=max(1+abs(dif))*2
          if(sum(dif>0)>0){
            indx=round(which.min(abs(dif+top*(dif<0)))+starts[bl])
            bracket[indx]=TRUE
          }
          if(sum(dif<0)>0){
            indx=round(which.min(abs(dif+top*(dif>0)))+starts[bl])
            bracket[indx]=TRUE
          }
        }
      }
      tmp=sort(unique(round(c(tmp,(1:n)[bracket]))))
      niter=length(tmp)
      if(niter==nprev) {
        ## stop if we don't add any new observations
        goon=FALSE
      }
    }
    recent=sort(unique(round(c(recent,tmp))))
  }
  return(list(select=recent,n.orig=n,n.sel=length(recent),n.iter=i))
}

