\name{spdest.cvr}
\alias{spdest.cvr}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ spdest.cvr }
\description{ Calculates the covariance matrix of the observed intraday option prices depending on the time of the transactions.}
\usage{
spdest.cvr(ttime, L, xd, nc, np, CL, PT)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{ttime}{ vector (of length n), \code{ttime} are the transaction times }
  \item{L}{ scalar, \code{L} is the number of distinct strike prices }
  \item{xd}{ matrix (n x L), \code{xd} is the design matrix }
  \item{nc}{ scalar, \code{nc} is the number of call option prices }
  \item{np}{ scalar, \code{np} is the number of put option prices }
  \item{CL}{ indicator of the presence of call option prices \code{CL} }
  \item{PT}{indicator of the presence of put option prices \code{PT} }
}
\details{
}
\value{
  \item{cvr }{the covariance matrix}
}
\references{  See Hrdle, W., Hlvka, Z. (2005) ,"Dynamics of state price densities", Discussion Paper 2005-021, SFB 649, Humboldt-Universitt zu Berlin. http://sfb649.wiwi.hu-berlin.de/ }
\author{ Zdenek Hlavka, Charles University in Prague, Sokolovska 83, 18675 Praha 8, Czech Republic, hlavka@karlin.mff.cuni.cz}
\note{ spdest.cvr() is usually called only within spdest() }

\seealso{ \code{\link{spdest}}, \code{\link{spdest.xd}} }
\examples{
library(MASS)
library(SPDest)

## set the random seed (change it to obtain different simulated values :-)
set.seed(1)

## set the number of observations
npoz=20

## set the true call and put prices and corresponding SPD
true.call=cbind(c(10,20,30,40),c(17.5,9,2.5,2))
true.put=cbind(c(10,20,30,40),c(3,4.5,8,17.5))
true.spd=c(0.15,0.2,0.6,0.05)

x=cbind(rbind(true.call,true.put),c(rep(0,4),rep(1,4)))

## calculate the nr. of replication of each strike and random order of transactions
rep.sim=rmultinom(1,npoz,rep(true.spd/2,2))
x.sim=apply(x,2,rep,times=rep.sim)
xord=order(x.sim[,3],x.sim[,1])
x.sim=x.sim[xord,]

## true covariance matrix for the random errors
temp=spdest.xd(x.sim)
ttime=sqrt(runif(npoz))

## true covariance matrix for the random errors using function spdest.cvr()
true.var=spdest.cvr(ttime,temp$L,temp$xd,nc=npoz-sum(x.sim[,3]),np=sum(x.sim[,3]),CL=TRUE,PT=TRUE)

## print the true covariance (calculated using spdest.cvr())
print(true.var)
}
\keyword{ regression }% at least one, from doc/KEYWORDS

