\name{spdest.xd}
\alias{spdest.xd}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ spdest.xd }
\description{
  calculates the design matrix for spdest()
}
\usage{
spdest.xd(price)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{price}{ Matrix (n x 3) \code{price} 1st column contains the strike price, 2nd column the corresponding option price and 3 column is (binary) indicator of put option price. }
}
\details{
}
\value{
  \item{y }{modified response variable (for the dependencies between call and put options)}
  \item{xc }{the part of xd corresp. to call options}
  \item{xp }{the part of xd corresp. to put options}
  \item{xd }{the design matrix}
  \item{L }{nr. of distinct strike prices}
  \item{K }{distinct strike prices}
}
\references{  See Hrdle, W., Hlvka, Z. (2005) ,"Dynamics of state price densities", Discussion Paper 2005-021, SFB 649, Humboldt-Universitt zu Berlin. http://sfb649.wiwi.hu-berlin.de/ }
\author{ Zdenek Hlavka, Charles University in Prague, Sokolovska 83, 18675 Praha 8, Czech Republic, hlavka@karlin.mff.cuni.cz}
\note{ spdest.xd() is usually called only within spdest() }

\seealso{ \code{\link{spdest}}, \code{\link{spdest.cvr}} }
\examples{
library(MASS)
library(SPDest)

## set the random seed (change it to obtain different simulated values :-)
set.seed(1)

## set the number of observations
npoz=20

## set the true call and put prices and corresponding SPD
true.call=cbind(c(10,20,30,40),c(17.5,9,2.5,2))
true.put=cbind(c(10,20,30,40),c(3,4.5,8,17.5))
true.spd=c(0.15,0.2,0.6,0.05)

x=cbind(rbind(true.call,true.put),c(rep(0,4),rep(1,4)))

## calculate the nr. of replication of each strike and random order of transactions
rep.sim=rmultinom(1,npoz,rep(true.spd/2,2))
x.sim=apply(x,2,rep,times=rep.sim)
xord=order(x.sim[,3],x.sim[,1])
x.sim=x.sim[xord,]

## calculate the design matrix for the experiment
temp=spdest.xd(x.sim)

## print the new response variable
print(temp$y)

## print the design matrix 
print(temp$xd)
}
\keyword{ regression }% at least one, from doc/KEYWORDS

